import {
  Card,
  Table,
  TableBody,
  TableCell,
  TableCellLayout,
  TableHeader,
  TableHeaderCell,
  TableRow,
  Title3,
} from "@fluentui/react-components";
import { Desktop24Regular } from "@fluentui/react-icons";
import { filesize } from "filesize";
import { Rights, VMGroup } from "../api/ServerApi";
import { VMState } from "../api/VMApi";
import React from "react";
import { GroupApi, GroupVMState } from "../api/GroupApi";
import { useToast } from "../hooks/providers/ToastProvider";

export function GroupsWidget(p: { rights: Rights }): React.ReactElement {
  return (
    <>
      {p.rights.groups.map((g) => (
        <GroupInfo group={g} />
      ))}
    </>
  );
}

function GroupInfo(p: { group: VMGroup }): React.ReactElement {
  const toast = useToast();

  const [state, setState] = React.useState<GroupVMState | undefined>();

  const load = async () => {
    const newState = await GroupApi.State(p.group);
    if (state !== newState) setState(newState);
  };

  React.useEffect(() => {
    const interval = setInterval(async () => {
      try {
        if (p.group.can_get_state) await load();
      } catch (e) {
        console.error(e);
        toast(
          "Error",
          `Failed to refresh group ${p.group.id} VMs status!`,
          "error"
        );
      }
    }, 1000);
    return () => clearInterval(interval);
  });

  return (
    <Card
      style={{
        margin: "50px 10px",
        display: "flex",
        flexDirection: "column",
      }}
    >
      <Title3 style={{ marginLeft: "10px" }}>{p.group.id}</Title3>
      <Table arial-label="Default table" style={{ minWidth: "510px" }}>
        <TableHeader>
          <TableRow>
            <TableHeaderCell>Name</TableHeaderCell>
            <TableHeaderCell>Description</TableHeaderCell>
            <TableHeaderCell>Resources</TableHeaderCell>
            <TableHeaderCell>State</TableHeaderCell>
            <TableHeaderCell>Actions</TableHeaderCell>
          </TableRow>
        </TableHeader>
        <TableBody>
          {p.group.vms.map((item) => (
            <TableRow key={item.uuid}>
              <TableCell>
                <TableCellLayout media={<Desktop24Regular />}>
                  {item.name}
                </TableCellLayout>
              </TableCell>
              <TableCell>{item.description}</TableCell>
              <TableCell>
                {item.architecture} &bull; RAM :{" "}
                {filesize(item.memory * 1000 * 1000)} &bull; {item.number_vcpu}{" "}
                vCPU
              </TableCell>
              <TableCell>{state?.[item.uuid] ?? ""}</TableCell>
            </TableRow>
          ))}
        </TableBody>
      </Table>
    </Card>
  );
}
